package com.unitesk.requality.eclipse.handlers.requirement;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.views.properties.PropertySheet;

import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.views.RequalityCNF;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class OpenRequirementHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        Requirement req = (Requirement) RequalityCNF.getSelectedNode();
        if (req.getLocationQIds().size() == 0) {
            return null;
        }
        Location loc = req.getLocation(req.getLocationQIds().get(0));
        NodeEditorInput input = new NodeEditorInput(loc.getParent());
        input.setData(loc);
        try {
            IEditorReference eref = Activator.findMarkEditorForDocument(loc
                    .getParent().getQualifiedId());
            if (eref == null) {
                PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                        .getActivePage().openEditor(input, ReqMarker.ID);
            } else {
                ((ReqMarker) eref.getEditor(true)).setTargetLocation(loc);
                eref.getPage().bringToTop(eref.getEditor(true));
            }
        } catch (PartInitException e) {
            e.printStackTrace();
        }
        showProperties();
        return null;
    }

    public static void showProperties() {
        PropertySheet props = (PropertySheet) PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage()
                .findView("org.eclipse.ui.views.PropertySheet");
        if (props == null) {
            try {
                PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                        .getActivePage()
                        .showView("org.eclipse.ui.views.PropertySheet");
            } catch (PartInitException e) {
                e.printStackTrace();
            }
            return;
        }

        boolean hasUnpinned = false;
        IViewReference[] viewReferences = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage().getViewReferences();
        for (int i = 0; i < viewReferences.length; i++) {
            if ("org.eclipse.ui.views.PropertySheet".equals(viewReferences[i]
                    .getId())) {
                PropertySheet propertyView = (PropertySheet) viewReferences[i]
                        .getView(true);
                if (!propertyView.isPinned()) {
                    hasUnpinned = true;
                    break;
                }
            }
        }
        if (!hasUnpinned) {
            IHandlerService handlerService = (IHandlerService) PlatformUI
                    .getWorkbench().getActiveWorkbenchWindow()
                    .getService(IHandlerService.class);
            try {
                handlerService
                        .executeCommand(
                                "org.eclipse.ui.views.properties.NewPropertySheetCommand",
                                null);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

}
