package com.unitesk.requality.eclipse.views;

import java.util.Comparator;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.tools.TreesTracker;

public class CNFContentProvider implements ITreeContentProvider {
    @Override
    public void dispose() {
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
    }

    @Override
    public Object[] getElements(Object inputElement) {
        if (inputElement instanceof IWorkspaceRoot) {
            return ResourcesPlugin.getWorkspace().getRoot().getProjects();
        }
        if (inputElement instanceof TreeNode) {
            TreeNode node = (TreeNode) inputElement;

            return node.getSortedChildren();
        }
        return null;
    }

    @Override
    public Object[] getChildren(Object parentElement) {
        Comparator<TreeNode> typeIdComp = new Comparator<TreeNode>() {
            @Override
            public int compare(TreeNode o1, TreeNode o2) {
                TreeNode n1 = (TreeNode) o1;
                TreeNode n2 = (TreeNode) o2;
                if (n1.getType().compareTo(n2.getType()) != 0) {
                    return n1.getType().compareTo(n2.getType());
                }
                return n1.getId().compareTo(n2.getId());
            }
        };
        if (parentElement instanceof IProject) {
            IProject project = (IProject) parentElement;
            if (TreesTracker.getTracker().getTree(project) != null
                    && TreesTracker.getTracker().getTree(project)
                            .getRootNode(false) != null // Root node can be not
                                                        // created yet
            ) {
                TreeNode node = TreesTracker.getTracker().getTree(project)
                        .getRootNode();
                return node.getSortedChildren(typeIdComp, null);
            }
            return new Object[0];
        }
        if (parentElement instanceof TreeNode) {
            TreeNode node = (TreeNode) parentElement;
            return node.getSortedChildren(typeIdComp, null);
        }
        return null;
    }

    @Override
    public Object getParent(Object element) {
        if (element instanceof TreeNode) {
            TreeNode node = (TreeNode) element;
            if (node.getTreeDB().getRootNode().equals(node.getParent())) {
                return TreesTracker.getTracker().getProject(node.getTreeDB());
            }
            return node.getParent();
        }
        return null;
    }

    @Override
    public boolean hasChildren(Object element) {
        return getChildren(element).length > 0;
    }
}
