package com.unitesk.requality.merge;

import org.w3c.dom.NamedNodeMap;

public class Chunk {

    private Node spanNode;
    private String locationText;
    private Node anchorNode;
    private String id;

    @SuppressWarnings("serial")
    public class InvalidLocationChunkException extends Exception {

    }

    public Chunk(Node x) throws InvalidLocationChunkException {
	if (!isLocation(x))
	    throw new InvalidLocationChunkException();
	spanNode = x;
	id = retrieveLocationId(x);
	anchorNode = retrieveAnchorNode(spanNode);
	locationText = anchorNode.getDomNode().getNodeValue();
    }

    public Chunk(Chunk old, Node newAnchor) {
	spanNode = old.spanNode;
	id = old.id;
	locationText = old.locationText;
	anchorNode = newAnchor;
    }

    private Node retrieveAnchorNode(Node span)
	    throws InvalidLocationChunkException {
	for (Node x : span.getChilds())
	    if (x.isText())
		return x;
	throw new InvalidLocationChunkException();
    }

    private String retrieveLocationId(Node x) {
	org.w3c.dom.Node atr = x.getDomNode().getAttributes()
		.getNamedItem("class");
	String atrValue = atr.getNodeValue();
	String id = atrValue.substring(atrValue.indexOf("id_") + 3);
	return id;
    }

    public String getLocationId() {
	return id;
    }

    public Node getSpanNode() {
	return spanNode;
    }

    public static boolean isLocation(Node n) {
	org.w3c.dom.Node x = n.getDomNode();
	NamedNodeMap attrs = x.getAttributes();
	if (attrs == null)
	    return false;
	org.w3c.dom.Node classAtt = attrs.getNamedItem("class");
	if (classAtt == null)
	    return false;
	String attValue = classAtt.getNodeValue();
	if (!attValue.contains("requality_text"))
	    return false;
	return true;
    }

    public String toString() {
	return this.id + " : " + this.locationText;
    }

    public Node getAnchorNode() {
	return anchorNode;
    }

    public String getChunkText() {
	return this.locationText;
    }
}
